#ifndef FLOCKHEADER
#define FLOCKHEADER

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <set>
#include <vector>
#include <map>

#include "PointLight.h"

#include "RenderableObject.h"
typedef struct
{
	double x;
	double y;
} vec2f;


typedef enum
{
	FFIRST = -1,
	Falignment,
	Fcohesion,
	Fseparation,
	//Favoidance,
	Fwander,
	Fseek,
	//Fboost,
	//Fwind,
	Fdrag,
	FLAST
} forceenum;

// agent info
typedef struct
{
	vec2f position;
	vec2f d_position[FLAST];

	vec2f velocity;
	vec2f d_velocity[FLAST];

	vec2f acceleration;
	vec2f d_acceleration[FLAST];

	vec2f forces[FLAST];
	vec2f d_forces[FLAST][FLAST];

	vec2f force;
	vec2f d_force[FLAST];

	double objective;
	double d_objective[FLAST];


	double objectiveN;
	double d_objectiveN[FLAST];

	double speed;
	double angle;

	int cell;
	bool avoid;

	double nFn;
	double view;

} agenttype;

//obstacle type
typedef struct
{
	vec2f position;
	double radius;
	int mincellX, mincellY, maxcellX, maxcellY;
} obstype;


class Flock : public RenderableObject
{
public:
	Flock();
	~Flock();
	void Render();
	double Step(double etime);
	void Init(int nagents, double s, double a, double p);
	void Reset();
	static void AllocStatic();
	static void FreeStatic();

	void PanView(double x, double y);
	void ZoomView(double x, double y);
	void SetAlpha(forceenum f, double weight);
	double GetAlpha(forceenum f);
	//the objective fn and derivatives
	double fobj;
	double d_fobj[FLAST];
	double GetTime();
private:

protected:

	bool learning;

	FILE * file;

	double ssigma;
	double asigma;
	double psigma;

	//the agent info
	int numAgents;
	agenttype *agents;
	agenttype *updateagents;

	//species attributes
	double mass;
	//double max_velocity;
	//double max_force;

	//neighbourhood definition
	double nradius;
	double nangle;
	double maxa;
	double mina;

	//force weights
	double alpha[FLAST];

	//force parameters
	double targspeed;

	//obstacle info
	int numObstacles;
	obstype * obstacles;

	//the seek point
	vec2f seekPos;
	
	//step number
	int step;
	double time;

	//the cell look up table
	typedef std::set<int>cstype;
	typedef std::vector<cstype > cvtype;
	cvtype cells;
	int cellsX,cellsY;

	//the circumferences of the toroidal space in meters
	double toroidX;
	double toroidY;

	//location of panning
	double panX;
	double panY;
	
	//size of the view window
	double viewW;
	double viewH;


	//display list for the agent model
	static GLuint agentDL;
	
	//gsl RNG stuff
	gsl_rng * rng;

};

#endif
