#ifndef SKELCHARHEADER
#define SKELCHARHEADER
#include "RenderableObject.h"
#include "Matrix4x4.h"
#include "Vector3.h"
#include "Quaternion.h"
#include "SkelCharInst.h"
#include <vector>
#include <string>

class SkelChar
{
	friend class SkelCharInst;
public:
	SkelChar();
	~SkelChar();
	
	
	static bool Load(const char * name, const char * file);
	static int CreateInstance(const char * name);
	static SkelCharInst * GetInstancePtr(const char * name, const unsigned int inst);
	static void Unload(const char * name);
	static void RetireInstance(const char * name, const unsigned int inst);
	
	int JointIndex(const char * name);

private:
protected:
	bool LoadSkel(const char* file);
	
	static std::map<std::string,SkelChar*> skels;
	
	friend class SkelCharInst;
	
	static int GUID;

	std::map<int,SkelCharInst*> instances;
	std::map<std::string,unsigned int> indexLUT;

	struct joint
	{
		std::string name; //the name of the joint
		Vector3 relpos; //relative position from parent joint
		Vector3 orient; //relative orientation from parent
		int parent; //index of parent
		Matrix4x4 Mrelative; //tranformation from parent
		Matrix4x4 Mabsolute; //transformation from root		
		Matrix4x4 Minverse; //inverse of absolute transform		

		Matrix4x4 MbindInv; //inverse of bind pose transform

		/*		
		Matrix4x4 Mfinal; // final skinning transform
		
		//the decomposed animation transform for the joint
		Matrix4x4 xRot;
		Vector3 xScale;
		Vector3 xTrans;
		Matrix4x4 xform;
		bool dirty; //if true, the joint needs to be updated on the next deform
*/
	};

	struct vertex
	{
		Vector3 bindpos; //the binding pose position
		unsigned int numjoints; //the number of joints acting on this vertex
		std::vector<Vector3> relpos; //the position relative to its parent bone
		std::vector<int> bone; //index to bones
		std::vector<float> weight; //bone weights
	};
	
	struct normal
	{
		Vector3 orient; //the orientation of the normal
		std::vector<Vector3> relorient; //orientation relative to bound joints
		int vertex; // one of the vertices that this normal is attached to
	};

	struct texcoord
	{
		float dat[2];
	};

	struct triangle
	{
		unsigned int verts[3]; //index to vertices
		unsigned int norms[3]; //index to normals
		unsigned int uvs[3]; //index to uv coords
	};

	
	std::vector<texcoord> uvs;
	std::vector<normal> norms; 
	std::vector<vertex> verts; 
	std::vector<triangle> polys;
	std::vector<joint> joints;
	
	unsigned int numVerts;
	unsigned int numPolys;
	unsigned int numJoints;
	unsigned int numNormals;
	unsigned int numUVs;
};
#endif