#include <Actor.h>
#include <tinyxml.h>

Actor::Actor()
{
	skelSet = false;
	defaultWindow = 500;
	matName = "";
	playing = false;
	blending = false;
	stillBlend = false;
}

Actor::~Actor()
{
}


void 
Actor::Draw()
{
	if (!skelSet)
		return;
	
	//Material::ApplyManagedMaterial(matName.c_str());
	
}


bool 
Actor::LoadXML(const char * name)
{
	TiXmlDocument doc;
	if (!doc.LoadFile(name))
	{
		printf("Error loading actor file: %s\n",doc.ErrorDesc());
		return false;
	}
	
	TiXmlNode* node = 0;
	TiXmlHandle docHandle( &doc );
	
	TiXmlElement* skelEl = docHandle.FirstChild( "actor" ).FirstChild( "skeleton" ).Element();

	if (!skelEl)
	{
		printf("Bad XML format - no skeleton node\n");
		return false;
	}
	TiXmlElement* animEl = docHandle.FirstChild( "actor" ).FirstChild( "animations" ).Element();

	if (!animEl)
	{
		printf("Bad XML format - no animations node\n");
		return false;
	}

	//iterate through each animation, loading it, and building up the attribute and action matrices

	TiXmlElement* child = animEl->FirstChildElement("anim");
	for( child; child; child=child->NextSiblingElement() )
	{
		printf("loading %s\n",child->Attribute("name"));
		if (!KeyFrameAnim::Load(child->Attribute("name"),child->Attribute("name")))
			return false;

	}


	return true;
}
bool 
Actor::SetSkel(const char * name)
{
	if (skelSet)
		SkelChar::RetireInstance(skelName.c_str(),skelInst);

	skelName = name;
	skelInst = SkelChar::CreateInstance(name);
	if (skelInst >= 0)
	{
		skelSet = true;
		SkelCharInst * sk = SkelChar::GetInstancePtr(skelName.c_str(),skelInst);
		savedPose = Pose(sk->GetNumJoints());
		currentPose = Pose(sk->GetNumJoints());
		SetPose(currentPose);
		AddChild(sk);
	}
	return (skelInst >= 0);
}

void 
Actor::SetPose(Pose & p)
{
	SkelCharInst * sk = SkelChar::GetInstancePtr(skelName.c_str(),skelInst);
	if (p.GetSize() != sk->GetNumJoints())
		return;
	currentPose = p;
	sk->SetPose(currentPose);
	sk->Deform();
}

bool 
Actor::PlayAnim(const char * name)
{
	if (!skelSet)
		return false;
	SkelCharInst * sk = SkelChar::GetInstancePtr(skelName.c_str(),skelInst);

	a1.startTime = TimeStamp();
	a1.name = name;
	a1.startxform = xform;
	GetLocalXform(a1.startxform);
	//a1.startPos = Pos;
	//a1.startRot = Angle;
	playing = true;
		
	//save the current pose
	savedPose = currentPose;
	savedPose.SetHeading(Quaternion());
	savedPose.SetPosition(Vector3(0,0,0));

	playing = true;
	stillBlend = true;
	return true;
}

void 
Actor::SetMaterial(const char * name)
{
	matName = name;
}

void 
Actor::Update()
{
	if (!skelSet)
		return;

	//if we're not playing an animation, there's nothing to do!
	if (!playing)
		return;

	Pose p1;
	TimeStamp now = TimeStamp();

	KeyFrameAnim * kf1;
	float elapsed1;

	SkelCharInst * sk = SkelChar::GetInstancePtr(skelName.c_str(),skelInst);

	//Get pose for anim1
	kf1 = KeyFrameAnim::GetPtr(a1.name.c_str());
	elapsed1 = (float)a1.startTime.Interval(now);
	kf1->SetTime(elapsed1*1000);
	p1 = kf1->GetPose();


	//figure out our interpolation value for the active blend
	float slerp = 0.0f;
	
	if (stillBlend)
	{
		float offset = (float)a1.startTime.Interval(now)*1000.0f;
		slerp = (offset) / defaultWindow;
	}

	//p1.AddTranslation(a1.startPos);
	//p1.AddBaseRotation(a1.startRot);
	//do the interpolation, if neccessary

	if (stillBlend)
		currentPose.LinearInterpolate(savedPose,p1,slerp);
	else 
		currentPose = p1;
	

	//check if done blending!
	if (slerp >= 1)
		stillBlend = false;

	// check if done animation 1 - wait for blend to end, then stop
	if (elapsed1*1000 >= kf1->GetDuration())
	{
		if (!stillBlend)
			playing = false;
	}

	//apply the current pose, and deform the skeleton
	sk->SetPose(currentPose);

	Vector3 v;
	currentPose.GetPosition(v);
	Quaternion q;
	currentPose.GetHeading(q);
	
	Matrix4x4 rm,tm;
	q.getMatrix(rm);
	tm.Translate(v);
	tm.PostMult(rm);
	tm.PreMult(a1.startxform);
	SetXform(tm);

	sk->Deform();
	
}

void 
Actor::GetJointXform(const char * name, Matrix4x4 & m)
{
	SkelCharInst * sk = SkelChar::GetInstancePtr(skelName.c_str(),skelInst);
	sk->GetJointXform(name,m);
}

