#include "PointLight.h"

PointLight::PointLight()
{
	//printf("Created new PointLight id %d \n",objID);
	diffuse[0] = diffuse[1] = diffuse[2] = diffuse[3] = 1;
	ambient[0] =ambient[1] = ambient[2] = 0;
	ambient[3] = 1;
	specular[0] =specular[1] = specular[2] = 0;
	specular[3] = 1;

	lightNum= 0;
	status = FALSE;
}

PointLight::~PointLight()
{
	Switch(FALSE);
}

void PointLight::SetDiffuse(GLfloat r,GLfloat g,GLfloat b)
{
	diffuse[0] = r;
	diffuse[1] = g;
	diffuse[2] = b;

	if (status)
		glLightfv(GL_LIGHT0+lightNum, GL_DIFFUSE, diffuse);

}

void PointLight::SetAmbient(GLfloat r,GLfloat g,GLfloat b)
{
	ambient[0] = r;
	ambient[1] = g;
	ambient[2] = b;

	if (status)
		glLightfv(GL_LIGHT0+lightNum, GL_AMBIENT, ambient);

}

void PointLight::SetSpecular(GLfloat r,GLfloat g,GLfloat b)
{
	specular[0] = r;
	specular[1] = g;
	specular[2] = b;

	if (status)
		glLightfv(GL_LIGHT0+lightNum, GL_SPECULAR, specular);
}


bool PointLight::Switch(bool onOff)
{
	if (!onOff && status)
	{
		//printf("PointLight %d turned off\n",objID);
		glDisable(GL_LIGHT0 + lightNum);
		status = false;
	}
	else if (!status && onOff)
	{
		for (int i = 0; i < 7;i++)
		{
			if (!glIsEnabled(GL_LIGHT0 + i))
			{
				glLightfv(GL_LIGHT0+i, GL_AMBIENT, ambient);
				glLightfv(GL_LIGHT0+i, GL_DIFFUSE, diffuse);
				glLightfv(GL_LIGHT0+i, GL_SPECULAR, specular);
				glLightfv(GL_LIGHT0+i, GL_POSITION,Pos.GetPtr());

				glEnable(GL_LIGHT0+i);

				lightNum = i;
				status = true;
				//printf("PointLight %d turned on as OGl light %d\n",objID,lightNum);
				break;
			}
			if (i == 7)
			{
				//printf ("Cant turn on PointLight %d - out of OGL lights\n",objID);
				return false; // no available lights
			}
		}
	}
	else
		glLightfv(GL_LIGHT0+lightNum, GL_POSITION,Pos.GetPtr());

	return true;
}

void PointLight::SetPos(GLfloat x,GLfloat y,GLfloat z)
{
	RenderableObject::SetPos(x,y,z);
	//glLightfv(GL_LIGHT0+lightNum, GL_POSITION,_Pos);
}

void PointLight::Draw()
{

	glLightfv(GL_LIGHT0+lightNum, GL_POSITION,Pos.GetPtr());

	bool lights = (bool)glIsEnabled(GL_LIGHTING);

	if (lights)
		glDisable(GL_LIGHTING);

	if (status)
		glColor3f(diffuse[0],diffuse[1],diffuse[2]);
	else
		glColor3f(.5f,.5f,.5f);
	
	glBegin(GL_TRIANGLES);
	
	glVertex3f(0,.125f,0);
	glVertex3f(.125f,0,-.125f);
	glVertex3f(-.125f,0,-.125f);
	
	glVertex3f(0,.125f,0);
	glVertex3f(0,0,.125f);
	glVertex3f(.125f,0,-.125f);

	glVertex3f(0,.125f,0);
	glVertex3f(-.125f,0,-.125f);
	glVertex3f(0,0,.125f);


	glVertex3f(0,-.125f,0);
	glVertex3f(-.125f,0,-.125f);
	glVertex3f(.125f,0,-.125f);
	
	glVertex3f(0,-.125f,0);
	glVertex3f(.125f,0,-.125f);
	glVertex3f(0,0,.125f);

	glVertex3f(0,-.125f,0);
	glVertex3f(0,0,.125f);
	glVertex3f(-.125f,0,-.125f);

	glEnd();

	if (lights)
		glEnable(GL_LIGHTING);

}
